# This file contains a list of all internal representations for Sym and more

class ListStart
end

class ListEnd
end

class Indexing
end

class DictStart
end

class DictEnd
end

# Seperator
class DictRelation
end

class Operator
end

class UniOperator
end

class Expression
end

class LParanthesis
end

class RParanthesis
end

class Terminator
end

class PrintOperator
end

class InputOperator
end

class AssignmentOperator
end

class EOF
end

class Variable
end

class Constant
end

class BreakOperator
end

class IncrementOperator
end


class ComparisonOperator
end

class ModifierOperator
end

class ListContainer
end

class HashContainer
end

class Conditional
end

class IfPart
end

class ElseIfPart
end

class ElsePart
end

class EndIfPart
end

class WhilePart
end

class EndWhilePart
end

class ForPart
end

class EndForPart
end

class FunctionCall
end

class Comma # ","
end

class ClassDef
end

class ClassEnd
end

class ClassName
end

class MethodCall
end

class ClassInit
end

class ReturnOperator
end

# Aliases :
class Int < Integer
end

class Double < Float
end
class Void
end

class Regexp
  def +(re)
    Regexp.new(self.to_s + re.to_s)
  end

  def *(arg)
    Regexp.new(self.to_s * arg)
  end
end

class Lexem
  attr_accessor :type, :val

  def initialize(type = nil, val = nil)
    @type = type
    @val = val

    if type == String
      @val = @val.slice(1..-2) if @val != nil
    end

  end
end

class SymDict
  attr_reader :dict
  def initialize(dict = {})
    @dict = dict
  end

  def print_self()
    if @dict.length == 0
      print "{}"
      return
    end

    print "{"
    index = 0
    for key in @dict.keys
      index += 1
      value = @dict[key]

      if key.class == SymDict or key.class == SymList
        key.print_self()
      else
        print key
      end
      print ":"

      if value.class == SymDict or value.class == SymList
        value.print_self()
      else
        print value
      end
      print ", " if index < @dict.length
    end
    print "}"
  end

  def get_value(key)
    raise "\nSYM-ERROR: Could not evaluate dictionary statement" if
                                                   not @dict.keys.include?(key)
    return @dict[key]
  end
end

class SymList
  attr_reader :list
  def initialize(list = [])
    @list = list
  end

  def print_self()
    if @list.length == 0
      print "[]"
      return
    end

    print "["
    for i in 0..@list.length
      element = @list[i]

      if element.class == SymList or element.class == SymDict
        element.print_self()
      else
        print element
      end
      print ", " if i < (@list.length-1)
    end
    print "]"
  end
  
end

class SymObject
  attr_reader :name, :variables, :methods
  attr_accessor :scope
  def initialize(name, vars, methods)
    @name = name        # Name of _class_, not the object
    @variables = vars
    @methods = methods

    s = {}
    for var_k in vars.keys
      s[var_k] = vars[var_k]
    end
    for met_k in methods.keys
      s[met_k] = methods[met_k]
    end

    @scope = Scope.new(s)
  end
end

class Class
  attr_reader :name, :variables, :methods
  def initialize(name, vars, methods)
    @name = name
    @variables = vars
    @methods = methods
  end
end

# Temporary function-definition for each lexem, the actual tag
class FunctionDef
  attr_reader :end, :code
  def initialize(end_def = false)
    @end = end_def # true if it's the end-tag
  end
end

# Internal function class, this is the actual sym-functions
class Function
  attr_reader :name, :returntype, :code, :arguments, :scope_ref
  def initialize(name, returntype, code, args, arg_scope)
    @name = name
    @returntype = returntype
    @code = code
    @arguments = args
    @scope_ref = arg_scope
  end

  def get_scope_ref()
    return @scope_ref
  end
end

# Generic type
class DataType
  attr_reader :datatype
  def initialize(type)
    @datatype = type
  end
end