#-*- coding:utf-8 -*-
require 'Backend/datatypes'

class Scope
  attr_accessor :storage
  
  def initialize(objects = {}, debug_mode = false)
    # Key: Name, as string
    # Value: Object reference
    @storage = objects
    @debug_mode = debug_mode
  end
  
  # Returns the object, from the name
  def get_name(name)
    return @storage[name]
  end

  # Returns the name, from the object
  def get_object_name(object)
    for key in @storage.keys
      return key if @storage[key] == object
    end
    
    return false
  end

  # Removes an object from storage
  def unstore_object(name, no_duplicate = false)
    raise "SYM-ERROR: Constant #{name} can't be deleted" if no_duplicate and
                    @storage.include?(name)

    puts "Scope: Deleting object #{name}" if @debug_mode
    @storage.delete(name)
  end
	

  # Arg 1: Name of object, as string
  # Arg 2: Object to be stored
  # Arg 3: true=raise an error if there is already something by that name
  def store_object(name, object, no_duplicate = false)
    raise "SYM-ERROR: Constant #{name} already in use" if no_duplicate and
                                                      @storage.include?(name)

    # Some names are reserved, these are not allowed to be stored
    not_allowed = ["return", "SYM_VERSION",
                                  "sym_current_recursion", "true", "false"]

    raise "SYM-ERROR : Can not create reserved identifier #{name}" if 
                                              not_allowed.include?(name)

    puts "Scope: Storing object #{name}" if @debug_mode
    @storage[name] = object
  end

  # Get's the scope as a string, for debugging purposes
  def to_s
    res = "Scope trace:\n"
    for s_key in @storage.keys
      res += "\t#{s_key}=>#{@storage[s_key]}\n"
    end
    return res
  end
end
