#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

if RUBY_VERSION.slice(0..2).to_f < 1.9
  puts "\nSym is incompatible with your ruby version (#{RUBY_VERSION}), " +
                                              "please upgrade to 1.9 or later\n"
  exit()
end

filename = ARGV[0]
debug = ARGV[1]

raise "SYM-ERROR: Invalid option specified."+
 "  Correct syntax is \"<Path to Symfile> [-d][--debug]\"" if not filename and not debug

# Debug-flag is allowed in several ways
# If it was not specified it will be nil
allowed = [nil, "d", "-d", "debug", "-debug", "--debug"]
raise "Invalid option '#{debug}'."+
  " Correct syntax is \"Symfile [-d][--debug]\"" if not allowed.include?(debug)

# ------ Main

require 'Backend/parser'

# Read the file
begin
	input = File.open(filename).read().to_s()

rescue Exception => e
	raise "SYM-ERROR: Could not find input file: #{filename}"
end
parser = SymParser.new(debug)

# Ignore warnings should they arise
$VERBOSE = nil

# Start parsing the code, catch the error if it's raised.
begin
  parser.parse(input)
rescue Exception => e
  puts "\n#{e}" if (/\ASYM-ERROR.*/m).match(e.to_s.strip)
  if not debug
    puts "An error has accured, do you wish to see backtrace log? [Y/N]"
    if STDIN.gets().strip.downcase == "y"
      parser = SymParser.new(true)
      parser.parse(input)
    end
  end
end
